#include <iostream>
#include "Symmetry.h"
#include <Qt/qstring.h>
#include <Qt/qprocess.h>

#define DEFAULT_SETTINGS_DIR "../../scripts/settings/"
#define DEFAULT_SRC_DIR "../../../data/pointDistr/"

class TestFunction2D
{
	public:
		TestFunction2D(QString typeFn);
		TestFunction2D(DeformCurve * pntsToUseAsSymSpace);
		void GetRange(double & minX, double & maxX, double & minY, double & maxY);
		double GetValue(double x, double y);
		QString m_sType;
};

FFTManager manager;	

void PrintUsage(char * programName)
{
	std::cout<<"Usage:"<<std::endl;
	std::cout<<programName<<" settings.txt outDir inDistr.pnts"<<std::endl;
	std::cout<<"\t\tor"<<std::endl;
	std::cout<<programName<<" pntsToAnalyze.pnts/TestName outDir sigma"<<std::endl;
	std::cout<<"\t\twhere valid test names: [TestIntegration]"<<std::endl;	
	std::cout<<"\t\tor"<<std::endl;
	std::cout<<programName<<" CCLloydTest outDir numPnts screenSize"<<std::endl;
	std::cout<<"\t\tor"<<std::endl;
	std::cout<<programName<<" TiledTest outDir numPnts screenSize"<<std::endl;
	std::cout<<"\t\tor"<<std::endl;
	std::cout<<programName<<" JitteredTest outDir numPnts screenSize noise"<<std::endl;
	exit(0);
}

void AnalyzeResult(QString file, QString outDir, double sigma);
void RunTest(QString testName, QString outDir);
void CCLloydTest(QString outDir, int numPnts, int screenSize, double sigma);
void TiledTest(QString outDir, int numPnts, int screenSize, double sigma);
void JitteredTest(QString outDir, int numPnts, int screenSize, double noise, double sigma);
void RandomTest(QString outDir, int numPnts, int screenSize, double sigma);
void TestErrors(char ** inDistributions, int numDistr);

int main(int argc, char** argv)
{
	if (argc==1)
		PrintUsage(argv[0]);
		
	if (QString(argv[1]).endsWith(".pnts") || QString(argv[1]).endsWith(".eps"))
		AnalyzeResult(argv[1], argv[2], atof(argv[3]));
	else if (QString(argv[1])=="CCLloydTest")
		CCLloydTest(argv[2], atoi(argv[4]), atoi(argv[5]), atof(argv[3]));
	else if (QString(argv[1])=="TiledTest")
		TiledTest(argv[2], atoi(argv[4]), atoi(argv[5]), atof(argv[3]));
	else if (QString(argv[1])=="JitteredTest")
		JitteredTest(argv[2], atoi(argv[4]), atoi(argv[5]), atof(argv[6]), atof(argv[3]));
	else if (QString(argv[1])=="RandomTest")
		RandomTest(argv[2], atoi(argv[4]), atoi(argv[5]), atof(argv[3]));
	else if (QString(argv[1])=="TestDistributionsOnSampling")
		TestErrors(&(argv[2]), argc-2);
	else if (argc==3)
		RunTest(argv[1], argv[2]);
	else if (argc>=4)
	{
		TimeProfiler profile;		
		QString settingsFile(argv[1]);
		if (argv[1][0]=='-')
			settingsFile = QString(DEFAULT_SETTINGS_DIR) + settingsFile.right(settingsFile.length()-1);
	//	std::cout<<"settings file="<<settingsFile.toStdString().c_str()<<std::endl;
		QString outDir(argv[2]);
		
		QString curveFile(argv[3]);
		if (curveFile.startsWith("-"))
			curveFile = QString(DEFAULT_SRC_DIR) + curveFile.right(curveFile.length()-1);

//		std::cout<<"Params="<<argv[0]<<"\n\t"<<argv[1]<<"\n\t"<<argv[2]<<"\n\t"<<argv[3]<<"\n\t"<<argv[4]
		
		std::cout<<"Loading parameters... "<<std::flush;	
		int argID = 4;
		SymParamsText paramsText(settingsFile, &(argv[argID]), argc-argID);
		SymPipelineParams * params = new SymPipelineParams(curveFile, paramsText);	// use filtered f as a target		
		QImage img = params->m_pDeformationModel->GetDeformedImage();
		std::cout<<"Initializing Assymetric Point Optimizer..."<<std::endl;
		SymPipeline pointDistributor(params);
		profile.startedProcess("Symmetrizing (optimization)");
		pointDistributor.FindOptimalSolution();
		profile.finishedProcess("Symmetrizing (optimization)", true);	
		
		pointDistributor.OutputResultsImg(outDir, &img, NULL, NULL, NULL, "png", true);	
		((DeformCurve*)params->m_pDeformationModel)->WriteCurve(outDir+"/Distribution.pnts");
		
//		//AnalyzeResult(outDir+"/Distribution.pnts", outDir, ((DeformCurve*)params->m_pDeformationModel)->m_fStdDev);
//		AnalyzeResult(outDir+"/Distribution.pnts", outDir, ((DeformCurve*)params->m_pDeformationModel)->m_fStdDev);
//		SymMatrix2DReal oldDeform(img);
//		SymMatrix2DComplex oldTrg(oldDeform.m, oldDeform.n);		
//		manager.FFT2D(oldDeform, oldTrg);		
//		//oldDeform.ToImage().save(outDir+"/Initial_FFT_f_.png");
//		oldTrg.ToImage().save(outDir+"/FFT_f.png");
		
	}
	else 
		assert(false);
}

void TestErrors(char ** inDistributions, int numDistr)
{
	for (int i=0; i<numDistr; i++)	//for each distribution
	{
		// for each function
	}
}

void CCLloydTest(QString outDir, int numPnts, int screenSize, double sigma)
{
	QProcess::execute("mkdir "+outDir);
	
	DeformCurve pnts(DEFAULT_SRC_DIR+QString("/ccvt_")+QString::number(numPnts)+"_"+QString::number(screenSize)+".eps", 1);
		//NOTE: points must be pre-generated and stored in /SymmetryFramework/data/pointDistr/ccvt_*.eps
		// where * is numPnts_screenDims
	assert(screenSize == pnts.m_ScreenWidth && screenSize == pnts.m_ScreenHeight);
	std::cout<<"veritces="<<(int)pnts.m_Vertices.size()<<std::endl;
	assert((int)pnts.m_Vertices.size()==numPnts);
	
	pnts.WriteCurve(outDir+"/Distribution.pnts");	
	AnalyzeResult(outDir+"/Distribution.pnts", outDir, sigma);		
}

void TiledTest(QString outDir, int numPnts, int screenSize, double sigma)
{
}

void RandomTest(QString outDir, int numPnts, int screenSize, double sigma)
{
	QProcess::execute("mkdir "+outDir);
	
	std::vector<DeformCurve::Vertex * > vertices;
	std::vector<DeformCurve::Vertex * > emptyVertices;	
	std::vector<DeformCurve::Edge * > emptyEdges;		
	srand(10248);
	for (int i=0; i<numPnts; i++)
	{
		DeformCurve::Vertex * newVertex = new DeformCurve::Vertex();
		double randVal = (double)rand() / (double)RAND_MAX;
		newVertex->pos[0] = randVal * screenSize;
		randVal = (double)rand() / (double)RAND_MAX;		
		newVertex->pos[1] = randVal * screenSize;
		vertices.push_back(newVertex);
	}
	
	DeformCurve pnts(emptyVertices, emptyEdges, vertices, 1, screenSize, screenSize, 2);		//arbitrary curve
	pnts.WriteCurve(outDir+"/Distribution.pnts");
	AnalyzeResult(outDir+"/Distribution.pnts", outDir, sigma);	
}

void JitteredTest(QString outDir, int numPnts, int screenSize, double noise, double sigma)
{
	QProcess::execute("mkdir "+outDir);
	
	assert(pow((int)sqrt(numPnts), 2)==numPnts);	//for now assume numPnts = some power
	int numX = sqrt(numPnts);
	double dx = (double)screenSize / (double)(numX+1);
	int randSeed = 10248;
	QString curveFile = DeformCurve::GetPointsDescriptor(dx, dx, numX, numX, 0, noise * dx, 0, 0, randSeed);
	QString curveFilename;		
	double * params = SymPipelineParams::DivideCurveParams(curveFile, &curveFilename);

	std::cout<<"Jittered Distribution: "<<numPnts<<" points on "<<screenSize<<" x "<<screenSize<<" with noise = "<<noise*dx<<" / "<<dx<<std::endl;
	DeformCurve::PointDistributionParams pntsParams;
	pntsParams.m_bSumGaussians = true;
	DeformCurve pnts(curveFilename, params, 1, screenSize, screenSize, 2, true, pntsParams);
	pnts.WriteCurve(outDir+"/Distribution.pnts");

	if (sigma!=0)
	{
		std::cout<<"Analysing"<<std::endl;
		AnalyzeResult(outDir+"/Distribution.pnts", outDir, sigma);
	}
}

void AnalyzeResult(QString file, QString outDir, double sigma)
{	
	if (file.startsWith("-"))
		file = QString(DEFAULT_SRC_DIR) + file.right(file.length()-1);
	
	std::cout<<"\n============"<<std::endl;
	std::cout<<"Analyzing "<<file.toStdString().c_str()<<std::endl;
	if (file.startsWith("-"))
		file = DEFAULT_SRC_DIR + file.right(file.length()-1);
//	std::cout<<"\tInitializing Curve"<<std::endl;
	
	DeformCurve::PointDistributionParams pntsParams;
	pntsParams.m_bSumGaussians = true;	
	if (sigma<0)
		sigma = 1.5;
		
	DeformCurve pnts(file, 1, -1, -1, sigma, true, pntsParams);
//	std::cout<<"\tDeforming Pnts"<<std::endl;
	SymMatrix2DReal & deformedBlured = (SymMatrix2DReal &)pnts.DeformAtCurrentHierarchyLevel(pnts.GetParameterVector());
	double L1f = 0;
	for (int i=0; i<deformedBlured.m; i++)
		for (int j=0; j<deformedBlured.n; j++)
			L1f += deformedBlured(i,j);
	deformedBlured.DivideBy(L1f/256.);
	
//	std::cout<<"\t\tDone! Getting image"<<std::endl;
	QImage notBluredImg = pnts.GetDeformedImage(2);
	notBluredImg.save(outDir+"/fp_img.png");
	
//	std::cout<<"\tPerforming FFT on "<<deformedBlured.m<<" x "<<deformedBlured.n<<" image "<<std::endl;		
	SymMatrix2DReal deformedNotBlured(notBluredImg);
	SymMatrix2DComplex trg(deformedNotBlured.m, deformedNotBlured.n);
	manager.FFT2D(deformedNotBlured, trg);
	SymMatrix2DReal realTrg(deformedNotBlured.m, deformedNotBlured.n);
	realTrg = trg;
	SymMatrix2DReal rearrangedTrg(deformedNotBlured.m, deformedNotBlured.n);
	realTrg.CopySquare(&rearrangedTrg, 0, 0, deformedNotBlured.m/2, deformedNotBlured.n/2, 
					   deformedNotBlured.m/2, deformedNotBlured.n/2);
	realTrg.CopySquare(&rearrangedTrg, 0, deformedNotBlured.n/2, deformedNotBlured.m/2, 0, 
					   deformedNotBlured.m/2, deformedNotBlured.n/2);
	realTrg.CopySquare(&rearrangedTrg, deformedNotBlured.m/2, 0, 0, deformedNotBlured.n/2, 
					   deformedNotBlured.m/2, deformedNotBlured.n/2);
	realTrg.CopySquare(&rearrangedTrg, deformedNotBlured.m/2, deformedNotBlured.n/2, 0, 0, 
					   deformedNotBlured.m/2, deformedNotBlured.n/2);

	double fftMax = rearrangedTrg(deformedNotBlured.m/2, deformedNotBlured.n/2);
	std::cout<<"\tmax(FFT(fp))="<<rearrangedTrg.GetMax(false)<<" min(FFT(fp))="<<rearrangedTrg.GetMin(false)<<std::endl;	
	rearrangedTrg(deformedNotBlured.m/2, deformedNotBlured.n/2) = 0;
//	std::cout<<"\t2ndmax(FFT(fp))="<<rearrangedTrg.GetMax(false)<<" 2ndmin(FFT(fp))="<<rearrangedTrg.GetMin(false)<<std::endl;		
	if (rearrangedTrg.GetMax(false)>80)
		std::cout<<"[WARNING]: rearrangeTrgMax="<<rearrangedTrg.GetMax(false)<<">90"<<std::endl;
	rearrangedTrg(deformedNotBlured.m/2, deformedNotBlured.n/2) = fftMax;
	rearrangedTrg.ClampToImage(0, 90).save(outDir+"/FFT_fp.png");
	
//	double minVal = log(rearrangedTrg.GetMin(false));
//	for (int i=0; i<rearrangedTrg.m; i++)
//		for (int j=0; j<rearrangedTrg.n; j++)
//		{
//			rearrangedTrg(i,j) = log(rearrangedTrg(i,j))-minVal;
//			if (i<2 && j<2)
//				std::cout<<"val["<<i<<", "<<j<<"] = "<<rearrangedTrg(i,j)<<std::endl;
//		}
//	rearrangedTrg.ToImage(true).save(outDir+"/FFT_fp.png");
	deformedBlured.ToImage(false).save(outDir+"/fp_blured.png");
	rearrangedTrg(deformedNotBlured.m/2, deformedNotBlured.n/2) = fftMax;
	
//	std::cout<<"\tCalculating symmetry space of "<<deformedBlured.m<<" x "<<deformedBlured.n<<" image "<<std::endl;		
	//SymSpaceTrans2D symSpace(deformedBlured.m, deformedBlured.n);	
	SymSpaceTransToroidal2D symSpace(deformedBlured.m, deformedBlured.n, 
									 deformedBlured.m-1, deformedBlured.n-1, SymSpace::NORM_NONE);	
	SymMatrix2DReal & symMatrix = (SymMatrix2DReal &)symSpace.CalculateSymmetryAndProcess(deformedBlured);	
	std::cout<<"\tmax(Sfp)="<<symMatrix.GetMax(false)<<" min(Sfp)="<<symMatrix.GetMin(false)<<std::endl;

//	SymMatrix2DReal centerOfSymmetry(7, 7);
//	symMatrix.CopySquare(&centerOfSymmetry, symMatrix.m/2-3, symMatrix.n/2-3, 0, 0, 7, 7);
//	for (int i=symMatrix.m/2-3; i<=symMatrix.m/2+3; i++)
//		for (int j=symMatrix.n/2-3; j<=symMatrix.n/2+3; j++)
//			symMatrix(i,j) = 0;
	symMatrix.ToImage(true).save(outDir+"/Sfp_unnorm.png");
	//assert(symMatrix.GetMin(false)>0.85);
	if (symMatrix.GetMin(false)<0.85)
		std::cout<<"[WARNING] : Min="<<symMatrix.GetMin(false)<<std::endl;
	symMatrix.ClampToImage(0.85, 1.3).save(outDir+"/Sfp.png");
	
	SymEnergySymmetrization energyFn(&symSpace, "LpNorm", true, 2);	
	double E = energyFn.GetEnergy(deformedBlured);
	std::cout<<"\tL2Energy="<<E<<". Scaled By "<<L1f<<": E="<<(E/(L1f*L1f))<<std::endl;
	std::cout<<"\tL2(Sf)="<<symMatrix.SumElements(2)<<" L1(Sf)="<<symMatrix.SumElements(1)<<std::endl;
	QFile analyzeFile(outDir+"/AnalysisReport.html");
	assert(analyzeFile.open(QIODevice::WriteOnly));
	QTextStream fileStream(&analyzeFile);
	fileStream<<"Vladimir Kim - Scrap Book.\n<br>This is just a scrap book, and should not be of interest to anyone except for my collaborators.<br><h4>Folder: Synth_12_1</h4>\n";
	fileStream<<"Reporting on Point Distribution\n";
	fileStream<<"<table border=1>\n";
	fileStream<<"<tr><td>Distribution</td>  <td>Gaussians</td>    <td>Fourier Space</td>	<td>Symmetry Space</td>  </tr> <tr>";
	fileStream<<"<td><img src=\"fp_img.png\">\n</td>";
	fileStream<<"<td><img src=\"fp_blured.png\">\n</td>";
	fileStream<<"<td><img src=\"FFT_fp.png\">\n</td>";
	fileStream<<"<td><img src=\"Sfp.png\">\n</td>";	
	fileStream<<"<\tr></table><br>";

	// Analyze FFT: 2 x 1D plots
	int centerX = rearrangedTrg.m/2;
	int centerY = rearrangedTrg.n/2;	
	int maxRadii = qMin(rearrangedTrg.m/2, rearrangedTrg.n/2)-1;
	double pi = 3.14159265;
	std::vector<double> Pr;
	std::vector<double> Sr;	
	
	bool fast = true;
	int scaleRSamples=1;
	int scaleTSamples=1;
	if (fast)
	{
		scaleRSamples = 1;
		scaleTSamples = 5;
		// Calculate Power and Anisotropy from discrete FFT
		int numRSamples=maxRadii*scaleRSamples;
		for (int ring=0; ring<numRSamples; ring++)
		{
			double arcLength=2. * pi * (double)ring/(double)scaleRSamples;
			int numTSamples = scaleTSamples*ceil(arcLength);
			double power=0;
			std::vector<double> values;
			std::vector<double> radii;
			for (int t=0; t<numTSamples; t++)
			{
				double r = (double)ring/(double)scaleRSamples;
				double x = centerX + r * cos((double)t*2.*pi / (double)numTSamples);
				double y = centerY + r * sin((double)t*2.*pi / (double)numTSamples);
				
				if (!((int)(x+0.5)>=0 && (int)(x+0.5)<rearrangedTrg.m
					&& (int)(y+0.5)>=0 && (int)(y+0.5)<rearrangedTrg.n))
				{
					std::cout<<"ring="<<ring<<std::endl;
					std::cout<<"r="<<r<<std::endl;
				}
				
				assert((int)(x+0.5)>=0 && (int)(x+0.5)<rearrangedTrg.m
					   && (int)(y+0.5)>=0 && (int)(y+0.5)<rearrangedTrg.n);
				double val = rearrangedTrg((int)(x+0.5), (int)(y+0.5));
				power += val;
				values.push_back(val);
				radii.push_back(r);
			}
			double mean = (power / values.size() * numTSamples) / arcLength;
			if (mean>0)
				//Pr.push_back(mean);
				Pr.push_back(20*log10(mean));
			else
				Pr.push_back(0);
			double stdDev2=0;		
			
			for (int i=0; i<(int)values.size(); i++)
				stdDev2 += (values[i]-mean)*(values[i]-mean)/arcLength;
			if (mean>0 && stdDev2>0)
				//Sr.push_back(stdDev2 / (mean*mean));
				Sr.push_back(20*log10(stdDev2 / (mean*mean)));
			else
				Sr.push_back(0);
		}
	}
	else
	{
		for (int r=0; r<maxRadii; r++)
		{
			double intervalRMin=r;
			double intervalRMax=r+1;
			
			double power=0;
			int numThetaSamples=250;
			for (int t=0; t<numThetaSamples; t++)
			{
				double theta = theta*2.*pi / (double)numThetaSamples;
				double cosTheta=cos(theta);
				double sinTheta=sin(theta);
				for (int j=0; j<(int)pnts.m_Vertices.size(); j++)
				{
					double xj = pnts.m_Vertices[j]->pos[0];
					double yj = pnts.m_Vertices[j]->pos[1];
					double B = xj*cosTheta + yj*sinTheta;
					power += r * sqrt(pow(cos(-r*B), 2)+pow(sin(-r*B), 2));
				}
			}
			Pr.push_back(20*log10(power));
		}
	}
	
	//		Power Spectrum
	fileStream<<"\n<br><b>Power Spectrum and Angular Anisotropy:</b><br>\nplot([";	
	for (int i=scaleRSamples; i<(int)Pr.size()/scaleRSamples; i++)
	{
		double val=0;
		for (int j=0; j<scaleRSamples; j++)
			val += Pr[i*scaleRSamples+j];
		val/=scaleRSamples;
		fileStream<<val<<" ";
	}
	sort(Pr.begin(), Pr.end());
	double scalePr = Pr[Pr.size()-1]; //Pr[Pr.size()-2]*2;
	fileStream<<"], 'LineWidth', 3); axis([0 "<<maxRadii<<" 0 "<<scalePr<<"]);";
	fileStream<<"set(gca,'FontSize',30, 'Box', 'off'); axis tight;";	//"title('Power Spectrum');\n";
	fileStream<<"L = get(gca,'XLim'); set(gca,'XTick',floor(linspace(L(1),L(2),3)));\n";
	fileStream<<"L = get(gca,'YLim'); set(gca,'YTick',floor(linspace(L(1),L(2),3)));\n";	
	
	//		Angular Anistropy		
	fileStream<<"\n<br>\nfigure,plot([";	
	for (int i=scaleRSamples; i<(int)Sr.size()/scaleRSamples; i++)
	{
		double val=0;
		for (int j=0; j<scaleRSamples; j++)
			val += Sr[i*scaleRSamples+j];
		val/=scaleRSamples;
		fileStream<<val<<" ";
	}
	fileStream<<"], 'LineWidth', 3);\n";//"title('Angular Anistropy');\n<br>";
	fileStream<<"set(gca,'FontSize',30, 'Box', 'off'); axis tight;";	//"title('Power Spectrum');\n";
	fileStream<<"L = get(gca,'XLim'); set(gca,'XTick',floor(linspace(L(1),L(2),2)));\n";
	fileStream<<"L = get(gca,'YLim'); set(gca,'YTick',floor(linspace(L(1),L(2),2)));\n";	
	
	fileStream<<"\n<br><br>Energy = "<<E<<"<br><br>\n";
	fileStream<<"\n(c) Vladimir Kim, Yaron Lipman, Thomas Funkhouser. Princeton University";
	fileStream.flush();

	pnts.WriteCurve(outDir+"/pnts.eps");
}

void IntegrationTest(QString outDir);
void FourierSpaceL2Test(SymMatrix2DReal & deformedMatrix);
void RunTest(QString testName, QString outDir)
{
	std::cout<<"Running test "<<testName.toStdString().c_str()<<std::endl;
	if (testName=="IntegrationTest")
		IntegrationTest(outDir);
}

void IntegrationTest(QString outDir)
{
	outDir+="/";
	QProcess::execute("mkdir "+outDir);
//	NumericalMethods::TestIntegration();
//	exit(0);
	
	double sigma = 1;
	double noise=0;
	QString curveFilename;	
	//dx, dy, numX, numY, numAdditional, highFreq, lowFreq, probMissing, seed, 
	QString curveFile = DeformCurve::GetPointsDescriptor(10, 10, 10, 10, 0, noise, 0, 0, 3925);
	double * params = SymPipelineParams::DivideCurveParams(curveFile, &curveFilename);		
	
	DeformCurve::PointDistributionParams pntsFast;
	pntsFast.m_bPointDistrSpeedup = true;
	pntsFast.m_bSumGaussians = true;
	pntsFast.m_iPointDistrDimX = 10;	//r
	pntsFast.m_iPointDistrDimY = 10;	//theta
	pntsFast.m_PointDistrPolyPower = 0;	//m = 2*p+1
	pntsFast.m_IntegrationType = DeformCurve::POLAR_COORDS_WITH_POLYNOMIAL;	//POLAR_COORDS_WITH_POLYNOMIAL POLAR_COORDS CARTESIAN_COORDS

	DeformCurve::PointDistributionParams pntsSlow;
	pntsSlow.m_bSumGaussians = true;
	
//	DeformCurve pnts1("../REGULAR_SlowPadded/VarSigma/sigma_1/Distribution.pnts", 1, -1, -1, sigma, true, pntsSlow);
//	DeformCurve pnts2("../REGULAR_SlowPadded/VarSigma/sigma_1/Distribution.pnts", 1, -1, -1, sigma, true, pntsFast);
	DeformCurve pnts1("points", params, 1, -1, -1, sigma, true, pntsSlow);
	DeformCurve pnts2("points", params, 1, -1, -1, sigma, true, pntsFast);

	int numVertices = pnts1.GetParameterVector().m/2;
//	int vertex = numVertices/2 + 5;
	int vertex = 8;	//22		//find [8,8] 
	int coordID = 0;
	
//	pnts2.SaveImages("", 512, 512);
//	exit(0);
	pnts1.DeformAtCurrentHierarchyLevel(pnts1.GetParameterVector());	
	
	double paramX = pnts1.GetParameterVector()(2*vertex+coordID);
	double dx = 1;
	SymMatrix2DReal & deformMatrix = (SymMatrix2DReal & ) pnts1.DeformAtCurrentHierarchyLevel(pnts1.GetParameterVector());
	SymSpaceTransToroidal2D symSpace(deformMatrix.m, deformMatrix.n, deformMatrix.m, deformMatrix.n, SymSpace::NORM_NONE);
	//SymSpaceTrans2D symSpace(deformMatrix.m, deformMatrix.n, 2*deformMatrix.m, 2*deformMatrix.n, SymSpace::NORM_NONE);
	
	SymEnergySymmetrization energyFn(&symSpace, "LpNorm", true, 2);	
	
	SymMatrix2DReal & Stmp = (SymMatrix2DReal &)symSpace.CalculateSymmetryAndProcess(pnts1.DeformAtCurrentHierarchyLevel(pnts1.GetParameterVector()));
	Stmp.ToImage().save(outDir+"S0.png");
	SymMatrix2DReal S0(Stmp.m, Stmp.n);
	S0 = Stmp;
	
	TimeProfiler profiler;
	std::vector<double> energies1;
	std::vector<double> energies2;
	std::vector<double> paramsVals1;
	std::vector<double> paramsVals2;	
	std::vector<double> averageScale;
	double bestScale = 0;
	double minParamX = paramX - 20;
	double maxParamX = paramX + 20;
//	double minParamX = paramX-20;
//	double maxParamX = paramX-20;
	profiler.startedProcess("Original Method");
	
	bool loadPrecalculated = false;
	int id=0;
	
	for (double currParamX = minParamX; currParamX <= maxParamX; currParamX+=dx)
	{
		if (!loadPrecalculated)
		{
//			pnts1.GetParameterVector()(2*vertex+coordID) = currParamX;
//			pnts2.SaveImages("", 512, 512);
			double penalty;
			SymMatrix2DReal & deformMatrix1 = (SymMatrix2DReal&) pnts1.TakeStepInDimension(2*vertex+coordID, currParamX - pnts2.GetParameterVector()(2*vertex+coordID), penalty);
//			SymMatrix2DReal & deformMatrix1 = (SymMatrix2DReal&) pnts1.DeformAtCurrentHierarchyLevel(pnts1.GetParameterVector());
			double energy = energyFn.GetEnergy(deformMatrix1);
//			std::cout<<"filename="<<(outDir+"\img_"+QString::number(currParamX)+".png").toStdString()<<std::endl;
			deformMatrix1.ToImage().save(outDir+"f_"+QString::number(id)+".png");
			SymMatrix2DReal& Si = (SymMatrix2DReal&)symSpace.CalculateSymmetryAndProcess(deformMatrix1);
			Si.ToImage().save(outDir+"Sf_"+QString::number(id)+".png");
			Si.Subtract(S0);
			Si.ToImage(false).save(outDir+"dSf_"+QString::number(id)+".png");
			id++;
			
			energies1.push_back(energy);	//symmetry energy
			//energies1.push_back(deformMatrix1.SumElements(2));	//L2f
			//energies1.push_back(energyFn.GetEnergy(deformMatrix1)*pow(deformMatrix1.SumElements(2),2.));	//L2S
			std::cout<<"."<<std::flush;
		}
		paramsVals1.push_back(currParamX - paramX);
	}

	std::cout<<std::endl;
	profiler.finishedProcess("Original Method", false);	
	if (!loadPrecalculated)
	{
		SymVectorReal energies1Vec(energies1);
		energies1Vec.SaveSymVector("PointsEnergies.vec");
	}
	else
	{
		SymVectorReal energies1Vec("PointsEnergies.vec");
		energies1 = energies1Vec.ToVector();
	}
	pnts2.DeformAtCurrentHierarchyLevel(pnts2.GetParameterVector());
	
	profiler.startedProcess("New Method");		
	for (double currParamX = minParamX; currParamX <= maxParamX; currParamX+=dx)
	{
		break;
		double penalty;
		SymMatrix2DReal & deformMatrix2 = (SymMatrix2DReal&) pnts2.TakeStepInDimension(2*vertex+coordID, currParamX - pnts2.GetParameterVector()(2*vertex+coordID), penalty); 
//		pnts2.GetParameterVector()(2*vertex+0) = currParamX;
//		SymMatrix2DReal & deformMatrix2 = (SymMatrix2DReal&) pnts2.DeformAtCurrentHierarchyLevel(pnts2.GetParameterVector());
		double L2S = pnts2.m_fL2S;
		double L2f = pnts2.m_fL2f;
		
		energies2.push_back(deformMatrix2(0,0));
		//energies2.push_back(L2f);
		//energies2.push_back(L2S);
		if (energies1.size()>=energies2.size())
		{
			averageScale.push_back(energies1[energies2.size()-1] / energies2[energies2.size()-1]);
			bestScale += energies1[energies2.size()-1] / energies2[energies2.size()-1];
		}
		paramsVals2.push_back(currParamX - paramX);
		std::cout<<"."<<std::flush;
//		std::cout<<"E2(0) = "<<energies2[energies2.size()-1]<<std::endl;
	}
	std::cout<<std::endl;
	profiler.finishedProcess("New Method", false);		
	
	bestScale/=averageScale.size();
	double stdDev = 0;
	double error = 0;
	double errorNotAdjusted=0;
	for (int i=0; i<(int)averageScale.size(); i++)
	{
		stdDev += pow(averageScale[i]-bestScale, 2.);
		if (energies2.size()==averageScale.size() && energies1.size()==averageScale.size())
		{
			errorNotAdjusted += pow(energies2[i] - energies1[i], 2);
			error += pow(bestScale * energies2[i] - energies1[i], 2.);
		}
	}
	std::cout<<"Mean Scale="<<bestScale<<std::endl;
	std::cout<<"StdDev="<<sqrt(stdDev)<<std::endl;
	std::cout<<"RMSE_Adjusted = "<<error/energies2.size()<<" Not_adj="<<errorNotAdjusted/energies2.size()<<std::endl;
	
	std::cout<<"plot(["<<std::flush;
	for (int i=0; i<(int)paramsVals1.size(); i++)
		std::cout<<paramsVals1[i]<<" "<<std::flush;
	std::cout<<"], ["<<std::flush;
	for (int i=0; i<(int)energies1.size(); i++)
		std::cout<<energies1[i]<<" "<<std::flush;
	std::cout<<"]); title('Energy Orignal L2');"<<std::endl;
	
	std::cout<<"figure, plot(["<<std::flush;
	for (int i=0; i<(int)paramsVals2.size(); i++)
		std::cout<<paramsVals2[i]<<" "<<std::flush;
	std::cout<<"], ["<<std::flush;
	for (int i=0; i<(int)energies2.size(); i++)
		std::cout<<energies2[i]<<" "<<std::flush;
	std::cout<<"]); title('Energy New L2');"<<std::endl;
	
	std::cout<<"\n"<<profiler.profile().toStdString().c_str()<<std::endl;	
}

void FourierSpaceL2Test(SymMatrix2DReal & deformMatrix)
{
	SymMatrix2DComplex tempFourier(deformMatrix.m, deformMatrix.n);
	
	manager.FFT2D(deformMatrix, tempFourier);
	double L2FT = 0;
	for (int i=0; i<tempFourier.m; i++)
		for (int j=0; j<tempFourier.n; j++)
			L2FT += tempFourier(i,j).dat[0]*tempFourier(i,j).dat[0] + tempFourier(i,j).dat[1]*tempFourier(i,j).dat[1];		
	std::cout<<"L2(f(x,y)) = "<<deformMatrix.SumElements(2)<<std::endl;
	std::cout<<"L1(f(x,y)) = "<<deformMatrix.SumElements(1)<<std::endl;
	std::cout<<"L2(FT(f(x,y))="<<L2FT / (tempFourier.m*tempFourier.n)<<std::endl;
	deformMatrix.ToImage().save("pnts1_img.png");	
}


TestFunction2D::TestFunction2D(QString typeFn)
{
	m_sType = typeFn;
}

TestFunction2D::TestFunction2D(DeformCurve * pntsToUseAsSymSpace)
{
	//todo
}

double TestFunction2D::GetValue(double x, double y)
{
	if (m_sType=="ZonePlate")
		return sin(x*x + y*y);
	else
		return 0;
}


