#include <iostream>
#include "Symmetry.h"
#include <Qt/qstring.h>
#include "MVCImage.h"

#define DEFAULT_SRC_DIR "../../../data/"
#define DEFAULT_SETTINGS_DIR "../../scripts/settings/"

void PrintUsage(char * programName)
{
	std::cout<<"Usage: "<<programName<<" Tile textureFile outFile newWidth newHeight numXTiles numYTiles boundarySize"<<std::endl;
	std::cout<<"\tor "<<programName<<" ToroidalStitch textureFile outFile newWidth newHeight"<<std::endl;	
}

int main(int argc, char** argv)
{
	if (argc==1)
		PrintUsage(argv[0]);
	
	assert(argc>=6);
	QString inTexture(argv[2]);
	if (inTexture.startsWith("-"))
		inTexture = DEFAULT_SRC_DIR+inTexture.right(inTexture.length()-1);
	QImage tileableImage(inTexture);
	assert(tileableImage.width()>0 && tileableImage.height()>0);
	QString outTiled(argv[3]);		
	double newWidth = atof(argv[4]);
	if (newWidth<=1)
		newWidth = newWidth*tileableImage.width();
	double newHeight = atof(argv[5]);
	if (newHeight<=1)
		newHeight = newHeight*tileableImage.height();
	
	// blend tileable image
	QImage finalTexture = tileableImage.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);	
//	finalTexture.save("finalTexture.png");
	
	if (strcmp(argv[1], "Tile")==0)
	{
		// load images and params
		assert(argc==9);
		int numXTiles = atoi(argv[6]);
		int numYTiles = atoi(argv[7]);
		int boundarySize = atoi(argv[8]);
		std::cout<<"Tiling Texture: "<<inTexture.toStdString().c_str()<<" Scale=["<<newWidth<<", "<<newHeight<<"], ";
		std::cout<<"numTiles=["<<numXTiles<<", "<<numYTiles<<"], boundary="<<boundarySize<<std::endl;
		
		QImage scaledTexture = tileableImage.scaled(newWidth+boundarySize, newHeight+boundarySize, Qt::KeepAspectRatio, Qt::SmoothTransformation); 		
		
		for (int x=0; x<finalTexture.width(); x++)
			for (int y=0; y<finalTexture.height(); y++)
				finalTexture.setPixel(x, y, scaledTexture.pixel(x, y));
		
		int boundaryHeight = scaledTexture.height()-finalTexture.height();
//		std::cout<<"Boundary Height="<<boundaryHeight<<std::endl;
			// blend horizontal strip
		for (int x=0; x<finalTexture.width(); x++)
		{
			for (int y=0; y<boundaryHeight; y++)
			{
				double distToText1 = (double)y / (double)boundaryHeight;
//				std::cout<<"blended: ["<<x<<", "<<y<<"] / ["<<finalTexture.width()<<", "<<finalTexture.height()<<"]"<<std::endl;
//				std::cout<<"scaledTexture: ["<<x<<", "<<y+finalTexture.height()<<"] / ["<<scaledTexture.width()<<", "<<scaledTexture.height()<<"]"<<std::endl;
				QRgb c1 = finalTexture.pixel(x, y);
				QRgb c2 = scaledTexture.pixel(x, y+scaledTexture.height()-boundaryHeight);				
				double w1 = pow(distToText1, 1);
				double w2 = pow(1.-distToText1, 1);
				w1 /= (w1+w2);
				w2 /= (w1+w2);	
				QRgb c_ave = qRgb(w1*(double)qRed(c1)	+ w2*(double)qRed(c2), 
								  w1*(double)qGreen(c1)	+ w2*(double)qGreen(c2),
								  w1*(double)qBlue(c1)	+ w2*(double)qBlue(c2));
				finalTexture.setPixel(x, y, c_ave);
			}
		}

		int boundaryWidth = scaledTexture.width()-finalTexture.width();
//		std::cout<<"Boundary Height="<<boundaryWidth<<std::endl;
			// blend vertical strip		
		for (int x=0; x<boundaryWidth; x++)
		{
			for (int y=0; y<finalTexture.height(); y++)
			{
				double distToText1 = (double)x / (double)boundaryWidth;
				QRgb c1 = finalTexture.pixel(x, y);
				QRgb c2 = scaledTexture.pixel(x+scaledTexture.width()-boundaryWidth, y);				
				double w1 = pow(distToText1, 1);
				double w2 = pow(1.-distToText1, 1);
				w1 /= (w1+w2);
				w2 /= (w1+w2);				
				QRgb c_ave = qRgb(w1*(double)qRed(c1)+w2*(double)qRed(c2), 
								  w1*(double)qGreen(c1)+w2*(double)qGreen(c2),
								  w1*(double)qBlue(c1)+w2*(double)qBlue(c2));
				finalTexture.setPixel(x, y, c_ave);				
			}
		}
		scaledTexture = finalTexture;
		
		// tile image
		QImage newTexture((int)(scaledTexture.width()*numXTiles), (int)(scaledTexture.height()*numYTiles), QImage::Format_RGB32);
		for (int x=0; x<newTexture.width(); x++)
		{
			for (int y=0; y<newTexture.height(); y++)
			{
				int srcX = x % scaledTexture.width();
				int srcY = y % scaledTexture.height();
				newTexture.setPixel(x, y, scaledTexture.pixel(srcX, srcY));
			}
		}
		newTexture.save(outTiled);
	}
	else if (strcmp(argv[1], "ToroidalStitch")==0)
	{
		assert(argc==6);
		MVCImage mvcImg;
		QImage output = mvcImg.StitchTileToItself(finalTexture);
		output.save(outTiled);
	}
	else
		assert(false);
}

