/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim, Princeton University			   *
 *   Author: Vladimir Kim (kim_vladimir@yahoo.com, vk@princeton.edu)       *
 *   Research Supervisor: Tom Funkhouser (funk@cs.princeton.edu)           *
 *                                  ****                                   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __CVLIB_H
#define __CVLIB_H

#include <iostream>
#include <vector>
#include <Qt/qimage.h>
#include "opencv/cv.h"
#include "R2Shapes/R2Shapes.h"
#include "R3Shapes/R3Shapes.h"

struct SortableEntry
{
	SortableEntry(double val, int id, void * ptr = NULL);
	double m_fValue;
	int m_iID;
	void * m_pData;
};

bool SortingFunction(SortableEntry entry1, SortableEntry entry2);

class CVLibRoutines
{
	public:
		enum PointsToMeshMethod
		{
			TRIANGULATE_NAIVE,
			TRIANGULATE_DELAUNAY
		};
		enum SortingMeshVerticesMethod
		{
			SORTING_VERTICES_GEODESIC,
			SORTING_VERTICES_DECIMATION
		};
		static void FindFeaturePnts(QImage & img, int &numPnts, std::vector<double> & trg, 
									double minDist, bool addMorePnts);
		static void FindFeatureMesh(QImage img, int &numPnts, std::vector<double> & vertices, 
									std::vector<int> & triangles, 
									PointsToMeshMethod triangluationMethod = TRIANGULATE_DELAUNAY, 
									bool addMorePnts = false);
		static void GetDelaunayTriangulation(std::vector<double> & vertices, std::vector<int> & triangles);
		static void SortVerticesByImportance(QImage & img, std::vector<double> & vertices, 
											 std::vector<int> & triangles, 
											 SortingMeshVerticesMethod sortingMethod);
		static void SortVerticesByDecimation(QImage & img, std::vector<double> & vertices, 
											 std::vector<int> & triangles, std::vector<int> & vertexOrderOut);
		static void ImageBasedDecimation(QImage & img, std::vector<double> & vertices, 
										 std::vector<int> & triangles, int targetNumVertices);
		static R3Mesh * ConvertToR3Mesh(std::vector<double> & vertices, std::vector<int> & triangles);
		static void ConvertFromR3Mesh(R3Mesh * srcMesh, std::vector<double> & vertices, 
									  std::vector<int> & triangles);

		static R3Mesh * m_pTempMeshPointer;
	
	
	
		//static int PickNextVertexGivenEdge(int v1, int v2, double nx, double ny, std::vector<double> &vertices);
		//static void GetOutwardNormal(double v1x, double v1y, double v2x, double v2y, double v3x, double v3y, double *nx, double *ny);
		static void DrawPoints(QImage & img, std::vector<double> & vertices);
		static void DrawMesh(QImage & img, std::vector<double> & vertices, std::vector<int> & triangles);
		static void GetRidOfRedundant(std::vector<double> & vertices, std::vector<double> & edgeVerticesX, 
									  std::vector<double> & edgeVerticesY, double width, double height, double minDist);
		static void AddBoundaryVertices(std::vector<double> & vertices, std::vector<int> & specialVertexIDs, double width, double height, double minDist);
		static void ProjectToBoundaryVertices(std::vector<double> & vertices, double width, double height, double minDist);
		static void AddVertexToTriangles(int vertex, std::vector<int> & triangles, std::vector<double> & vertices, double minDist, double width, double height);
		static bool CheckInTriangle(double v1x, double v1y, double v2x, double v2y, double v3x, double v3y, double vx, double vy);
		static double DistanceToEdge(double v1x, double v1y, double v2x, double v2y, double vx, double vy);
		static double GetEdgeWeight(QImage & img, double v1x, double v1y, double v2x, double v2y);

};

#endif



