/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim, Princeton University			   *
 *   Author: Vladimir Kim (kim_vladimir@yahoo.com, vk@princeton.edu)       *
 *   Research Supervisor: Tom Funkhouser (funk@cs.princeton.edu)           *
 *                                  ****                                   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __DRAWING_ROUTINES_H
#define __DRAWING_ROUTINES_H

#include <iostream>
#include <vector>
#include <Qt/qimage.h>

/**
 * Superclass for a regular structure
 */
class DrawingRoutines
{
	public:
		enum DrawDirections
		{
			DRAW_OVER,
			ADD_INTENSITY,
			SUBTRACT_INTENSITY,
			TAKE_MAX,
			TAKE_MIN
		};
		
		static void DrawMaskOver(QImage & image, QImage & mask, QRgb emptyColor);
		static void DrawCircle(QImage & target, float x, float y, float radius, float width,
							   QRgb inside_color, QRgb outside_color, DrawDirections direct = DRAW_OVER);
		static void DrawEllipse(QImage & target, float x, float y, float radiusX, float radiusY,
								QRgb color, DrawDirections direct = DRAW_OVER);
		static void DrawLine(QImage & target, float x1, float y1, float x2, float y2,
							 float width, QRgb color, DrawDirections direct = DRAW_OVER);
		static void DrawRectangle(QImage & target, float x0, float y0, float w, float h, 
								  QRgb color, DrawDirections direct = DRAW_OVER);
//		static void DrawArrow(QImage & target, float x1, float y1, float x2, float y2,
//						  float width, QRgb color, DrawDirections direct = DRAW_OVER);
		static void DrawPixel(QImage & target, float x, float y, QRgb color, DrawDirections direct = DRAW_OVER);
		static void DrawPixel(QImage & target, int x, int y, QRgb color, DrawDirections direct = DRAW_OVER);	
		static void ClearImage(QImage & target, QRgb color);
};

#endif

