/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim, Princeton University			   *
 *   Author: Vladimir Kim (kim_vladimir@yahoo.com, vk@princeton.edu)       *
 *   Research Supervisor: Tom Funkhouser (funk@cs.princeton.edu)           *
 *                                  ****                                   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __MVC_IMAGE_H
#define __MVC_IMAGE_H

#include <iostream>
#include <vector>
#include <Qt/qimage.h>
#include <QtOpenGL/QGLContext>
#include <QtOpenGL/QGLFramebufferObject>
#include <QtOpenGL/QGLPixelBuffer>
#include <QtOpenGL/QGLFormat>
#include "opencv/cv.h"
#include "R2Shapes/R2Shapes.h"
#include "R3Shapes/R3Shapes.h" 

class DeformImageWarpLinearMesh;

class MVCMeshVertex
{
	public:
		MVCMeshVertex(double origX, double origY);
		std::vector<double> m_MVCoords;
		double x, y;
		double m_BoundaryDiffR;
		double m_BoundaryDiffG;	
		double m_BoundaryDiffB;		
};

/** Mean Value Coordinate Image Manipulation. 
	See Zeev Farbman, Gil Hoffer, Yaron Lipman, Daniel Cohen-Or, Dani Lischinski
	Coordinates for Instant Image Cloning, 
	Proceedings of Siggraph 2008. */
class MVCImage
{
	public:
		MVCImage();
		QImage StitchTileToItself(QImage tile);
		
		QImage InsertSrcAt(int x, int y);	
		QImage InsertMembraneAt(QImage membrane, int x, int y);	
		
		std::vector<int> CreateRectangleBoundary(int numBnd);
		void OverlayRectangleMesh(std::vector<int> & boundary, int numX, int numY);
		void CalculateMVC();

		QImage RenderMeshMVCInterpolant();
		void InitializeBuffer();
	
	protected:
		int NUM_GRID_PNTS;
		int NUM_BND_PNTS;
		QImage m_TrgImg;
		QImage m_SrcImg;
		std::vector<int> m_Boundary;	//in src coords
		std::vector<MVCMeshVertex> m_MeshVertices;
		std::vector<int> m_Faces;
		
		QGLPixelBuffer * m_pFramebuffer;
		int m_iTempCoord;
};

#endif



