#include "Numerical.h"
#include "Deformations.h"
#include "R2Shapes/R2Shapes.h"
#include "R3Shapes/R3Shapes.h"
#include "R3Graphics/R3Graphics.h"
#include <QtOpenGL/QGLContext>
#include <QtOpenGL/QGLFramebufferObject>
#include <QtOpenGL/QGLPixelBuffer>
#include <QtOpenGL/QGLFormat>
#include <Qt/qimage.h>


#ifndef __DEFORM_IMAGE_WARPING_H
#define __DEFORM_IMAGE_WARPING_H
class ImageWarpLinearMesh;

class ImageWarpTestExample : public ExampleWithAnswer
{
	public:
		ImageWarpTestExample(QString name);
		void GenerateExample(int randomSeed);
		void AddGlyph(QImage &img, double locX, double locY, double color, double dimX, double dimY);
		ImageWarpLinearMesh * CreateDeformationModel(QImage & inImg, std::vector<double> & featurePnts, int numRigidControls);
		void PrintParams();
	
		//params for perfect
			//lattice
		double m_BackgroundColor;	
		int m_NumX;
		int m_NumY;
		double m_Blur;
		R2Vector m_V1;
		R2Vector m_V2;
		double m_OddShift;
	
			//glyphs
		QString m_GlyphName;
		double m_GlyphX;
		double m_GlyphY;
		double m_GlyphMeanColor;
		bool m_bPutCtrlAtGlyphCorners;	
		bool m_bHierarchical;
	
		//noise params
		double m_FractionMissing;					// 0 to 1
		double m_FractionExtra;						// 0 to 1
		int m_ExtraBlobs;							// any integer > 0
		double m_ExtraBlobsFractionArea;			// 0 to 1
		double m_GlyphLocationVariationHighFreq;	// O(V1 or V2)
		double m_GlyphLocationVariationLowFreq;		// O(V1 or V2)
		double m_GlyphDimensionsVariation;			// O(glyphX or glyphY)
		double m_GlyphColorVariations;				// 0 to 2
		double m_GlyphQuadVariations;
};

class ImageWarpExampleCheckerboard : public ExampleWithAnswer
{
	public:
		ImageWarpExampleCheckerboard(QString name);
		void GenerateExample(int randomSeed);
		void PrintParams();
		
		//params for perfect
		//lattice
		int m_NumX;
		int m_NumY;	
		int m_WidthX;
		int m_WidthY;	
		
		//noise
		double m_fNoise;
};


class ImageWarp : public DeformationModel
{
	public:
		struct ImagePenalization
		{
			ImagePenalization();
			enum PenaltyFields
			{
				FOLDOVER_BINARY = 1,
				FOLDOVER_SMOOTH = 2,
				STAY_ON_CANVAS = 4,
			};
			
			int m_iFields;
			//maybe more params?
		};
	
		struct ImgVertex
		{
			enum BoundaryFixed
			{
				BND_VERTEX_NOT_FIXED = 0,
				BND_VERTEX_FIXED_X = 1,
				BND_VERTEX_FIXED_Y = 2
			};
			
			int m_iParamID;
			double x, y;
			double tu, tv;
			double m_fAsRigidRestX;
			double m_fAsRigidRestY;
			double m_fBoundaryX;
			double m_fBoundaryY;			
			int m_BoundaryVertexFlags;
			std::vector<int> m_AdjacentFaces;
			std::vector<int> m_AdjacentVertices;			
			void AddAdjacentVertex(int vertex);
		};
	
		struct ImgFace
		{
			int v1, v2, v3, v4;
		};
	
		enum ImageHierarchyType
		{
			CONST_CHILDREN,	//simply another grid with fewer params, over the main grid
			OVERLAP_CHILDREN,
			AS_RIGID_AS_POSSIBLE,
			AS_RIGID_AS_POSSIBLE_SOFT_CONSTRAINTS
		};
	
		struct ImageHierarchyDescriptor
		{
			ImageHierarchyDescriptor();
			ImageHierarchyType m_HierarchyType;  
			int m_iAssocX;
			int m_iAssocY;
			int m_iOverlapX;
			int m_iOverlapY;
			int m_iMinPnts;
			int m_iMaxPnts;
			int m_iInitNumPntsX;
			int m_iInitNumPntsY;	
			
		};
	
		ImageWarp(QImage & img);
		virtual ~ImageWarp(){}
		virtual bool HasOutput(OutputType type);
	
		//Testing	
		static QImage TestCreateCheckerboard(int numCellsX, int numCellsY, int cellWidth, int cellHeight, 
											 double rndLevel, QString meshing = "tri1");
		static QImage TestCreateRegularSpacedGlyph(int numGlyphX, int numGlyphY, R2Vector step1, R2Vector step2, 
												   double glyphWidth, double glyphHeight, QString glyphType);
		
	protected:
		QImage m_Source;
		SymMatrix2DReal * m_pSourceMatrix;
		QImage m_Deformed;
};

#endif

