#include "Numerical.h"
#include "DeformImageWarping.h"
#include "R2Shapes/R2Shapes.h"
#include "R3Shapes/R3Shapes.h"
#include "R3Graphics/R3Graphics.h"
#include <QtOpenGL/QGLContext>
#include <QtOpenGL/QGLFramebufferObject>
#include <QtOpenGL/QGLPixelBuffer>
#include <QtOpenGL/QGLFormat>
#include <Qt/qimage.h>


#ifndef __DEFORM_LENS_DISTORTION_H
#define __DEFORM_LENS_DISTORTION_H

class DeformationLensDistortion : public ImageWarp
{
	public:
		enum CameraModel
		{
			TSAI_SIMPLE,
		};
	
		enum TsaiModelFlags
		{
			TSAI_kx	= 1,
			TSAI_ky	= 2,
			TSAI_cx = 4,
			TSAI_cy = 8,
			TSAI_Rx = 16,
			TSAI_Ry = 32,
			TSAI_Rz = 64,
			TSAI_Tx = 128,
			TSAI_Ty = 256,
			TSAI_Tz = 512,			
			TSAI_f	= 1024,
			TSAI_MAX = 11
		};
	
		DeformationLensDistortion(QImage & img, int maxWidth, int maxHeight, CameraModel model, int flags);
		virtual SymMatrix & Deform(SymVectorReal & params, double * outPenalty = NULL);
		virtual QImage GetDeformedImage(int imageType = 0);
		virtual void GetOutputDimensions(int * w, int * h);	
		virtual void SetPostFilter(SymFilter * filter);
		ImageWarpLinearMesh * m_pMesh;
	
		static void GetUnprojectedValue(double x, double y, double z, double * newX, double * newY, 
										double kx, double ky, double cx, double cy, double f, 
										double locSx, double locSy, double Rx, double Ry, double Rz, 
										double Tx, double Ty, double Tz );
		static void TestLensDistortion();
		CameraModel m_Model;	
		SymVectorReal * m_pOriginalParams;
		double m_fScaleX;
		double m_fScaleY;
		std::vector<int> m_FlagToParam;
		std::vector<int> m_Params;
};

#endif

