#include "SymEnergy.h"

/************************* SymEnergyDifference ******************************/
SymEnergyDifference::SymEnergyDifference(SymSpace * S, double p, SymMatrix2DReal & Sg)
: SymEnergy(S)
{
	assert(S!=NULL);
	this->p = p;
	m_pTargetSymmetry = &Sg;
//	m_pTargetSymmetry->ToImage().save("FILTER_initialSG.png");
}

SymEnergyDifference::~SymEnergyDifference()
{
}

double SymEnergyDifference::GetEnergy(SymMatrix & f)
{
//	std::cout<<"\n=================="<<std::endl;
	SymMatrix & Sf = m_pS->CalculateSymmetryAndProcess(f);
//	std::cout<<"Sf="<<((SymMatrix2DReal&)Sf).m<<" x "<<((SymMatrix2DReal&)Sf).n<<std::endl;
//	std::cout<<"Sg="<<((SymMatrix2DReal&)*m_pTargetSymmetry).m<<" x "<<((SymMatrix2DReal&)*m_pTargetSymmetry).n<<std::endl;	
	double val = ((SymMatrix2DReal&)Sf).CalculateDifference(*m_pTargetSymmetry, p);
//		((SymMatrix2DReal&)Sf).ToImage().save("Diff_sCurr.png");
//		((SymMatrix2DReal&)*m_pTargetSymmetry).ToImage().save("Diff_sTrg.png");			
//		std::cout<<"maxCurr="<<((SymMatrix2DReal&)Sf).GetMax(false)<<" maxTrg="<<((SymMatrix2DReal&)*m_pTargetSymmetry).GetMax(false)<<std::endl;
//		std::cout<<"minCurr="<<((SymMatrix2DReal&)Sf).GetMin(false)<<" minTrg="<<((SymMatrix2DReal&)*m_pTargetSymmetry).GetMin(false)<<std::endl;
//		std::cout<<"p="<<p<<std::endl;
//		std::cout<<"val="<<val<<std::endl;
//
//		SymMatrix2DReal * diffMatrix = new SymMatrix2DReal(((SymMatrix2DReal&)Sf).m, ((SymMatrix2DReal&)Sf).n);
//		for (int i=0; i<((SymMatrix2DReal&)Sf).m; i++)
//			for (int j=0; j<((SymMatrix2DReal&)Sf).n; j++)
//				(*diffMatrix)(i,j) = pow(((SymMatrix2DReal&)Sf)(i,j)-((SymMatrix2DReal&)*m_pTargetSymmetry)(i,j), 2);
//		diffMatrix->ToImage(false).save("Diff.png");
//		exit(0);
	return val;
}

/************************* SymEnergySymmetrization ******************************/
SymEnergySymmetrization::SymEnergySymmetrization(SymSpace * S, QString energyType, 
												 bool assymmetrization, double p)
: SymEnergy(S)
{
	m_sEnergyType = energyType;
	if (assymmetrization)
		m_fSign = 1;
	else
		m_fSign = -1;
	m_fP = p;
}

SymEnergySymmetrization::~SymEnergySymmetrization()
{
}


double SymEnergySymmetrization::GetEnergy(SymMatrix & f)
{
	//just to make sure that algorithm is all here
//	assert(m_pS->m_Norm == SymSpace::NORM_IMAGE_SPACE);	
	SymMatrix & Sf = m_pS->CalculateSymmetryAndProcess(f);
	if (m_sEnergyType=="RatioOfNormsWithOneInNominator")
	{
		return PeakinessL2OverL1With1InNom(Sf);
	}
	else if (m_sEnergyType=="RatioOfNorms")
	{
		return PeakinessL2OverL1(Sf);		
	}
	else if (m_sEnergyType=="SquaredDistancenFromMean")
	{
		return SumSquaresToMean(Sf);		
	}	
	else if (m_sEnergyType=="ColorConsistentSymmetrization")
	{
		assert(f.GetMax(true)<=1.);
		double peakiness1 = SumSquaresToMean(Sf);
		f.ConstMinusValue(1);		
		SymMatrix & Sf2 = m_pS->CalculateSymmetryAndProcess(f);
		f.ConstMinusValue(1);		
		return peakiness1 + SumSquaresToMean(Sf2);
		
//		assert(f.GetMax(true)<=1.);
//		double peakiness1 = PeakinessL2OverL1(Sf);
//		f.ConstMinusValue(1);		
//		SymMatrix & Sf2 = m_pS->CalculateSymmetryAndProcess(f);
//		f.ConstMinusValue(1);		
//		return peakiness1 + PeakinessL2OverL1(Sf2);
	}
	else if (m_sEnergyType=="LpNorm")
	{
		return m_fSign * Sf.SumElements(m_fP);
	}
	else if (m_sEnergyType=="LinfNorm")
	{
		return m_fSign * Sf.GetMax(true);
	}
	else if (m_sEnergyType=="GradientX")
	{
		return m_fSign * SumGradients(Sf, 0);
	}
	else if (m_sEnergyType=="GradientY")
	{
		return m_fSign * SumGradients(Sf, 1);
	}	
	else if (m_sEnergyType=="Sobel")
	{
		return m_fSign * SumEdges(Sf);
	}
	std::cout<<"Type="<<m_sEnergyType.toStdString().c_str()<<std::endl;
	assert(false);
	return NULL;
}

double SymEnergySymmetrization::PeakinessL2OverL1(SymMatrix & Sf)
{
	double mean = Sf.SumElements(1) / Sf.GetNumElements();
	Sf.DivideBy(mean);
	return m_fSign * Sf.SumElements(2);
}

double SymEnergySymmetrization::PeakinessL2OverL1With1InNom(SymMatrix & Sf)
{
	double mean = Sf.SumElements(1) / Sf.GetNumElements();
	Sf.Subtract(-1);	// add 1	
	Sf.DivideBy(mean);
	return m_fSign * Sf.SumElements(2);	
}

double SymEnergySymmetrization::SumSquaresToMean(SymMatrix & Sf)
{
	double mean = Sf.SumElements(1) / Sf.GetNumElements();
	Sf.Subtract(mean);
	return m_fSign * Sf.SumElements(2);			
//	assert(Sf.GetNumDims()==2 && Sf.GetEntryType()==SYM_ENTRY_REAL);
//	SymMatrix2DReal & SfReal = (SymMatrix2DReal &)Sf;
//	double E = 0;
//	for (int i=0; i<SfReal.m; i++)
//		for (int j=0; j<SfReal.n; j++)
//			if (SfReal(i,j) > 0)
//				E += SfReal(i,j)*SfReal(i,j);
//	return m_fSign * E;
}

double SymEnergySymmetrization::SumGradients(SymMatrix & Sf, int dim)
{
	assert(Sf.GetNumDims()==2 && Sf.GetEntryType()==SYM_ENTRY_REAL);
	SymMatrix2DReal & SfReal = (SymMatrix2DReal&)Sf;
	SfReal.FillGrid();
//	std::cout<<"[WARNING] Printing matrix"<<std::endl;
//	SfReal.ToImage().save("SymEnergyEdgesIn.png");
	SfReal.m_pTempFilterGrid->Gradient(dim);
	SfReal.CopyFromGrid();
	for (int i=0; i<SfReal.m; i++)
	{
		SfReal(i, 0) = 0;
		SfReal(i, SfReal.n-1) = 0;		
	}
	for (int i=0; i<SfReal.n; i++)
	{
		SfReal(0, i) = 0;
		SfReal(SfReal.m-1, i) = 0;		
	}
	
//	SfReal.ToImage().save("SymEnergyEdgesOut.png");	
//	exit(0);	
	return Sf.SumElements(2);
}

double SymEnergySymmetrization::SumEdges(SymMatrix & Sf)
{
	assert(Sf.GetNumDims()==2 && Sf.GetEntryType()==SYM_ENTRY_REAL);
	SymMatrix2DReal & SfReal = (SymMatrix2DReal&)Sf;
//	SfReal.FillGrid();
//	SfReal.m_pTempFilterGrid->Gradient(dim);
//	SfReal.CopyFromGrid();
//	SfReal.ToImage().save("SymEnergyEdgesIn.png");
	SfReal.SobelFilterOpenCV();
//	SfReal.ToImage().save("SymEnergyEdgesOut.png");
//	exit(0);
	return SfReal.SumElements(2);	
}


