#include "SymFilters.h"

SymFilter::SymFilter(QString specialDirections)
{
	m_sSpecialDirections=specialDirections;
}

void SymFilter::Process(SymMatrix * inMatrix, SymMatrix ** outMatrix)
{
	Process(inMatrix);
	*outMatrix = inMatrix;
}

void SymFilter::Process(SymMatrix * )
{
	assert(false);
}

SymFilterScript::SymFilterScript(QString excutable, QStringList params)
{
	m_sFilterType = excutable;
	m_sParameters = params.join(" ");
}

void SymFilterScript::Process(SymMatrix * inMatrix, SymMatrix ** outMatrix)
{
	std::cout<<"[WARNING] SymFilterScript::Process creates new matrix"<<std::endl;
	assert(inMatrix->GetNumDims()==2 && inMatrix->GetEntryType()==SYM_ENTRY_REAL);
	((SymMatrix2DReal*)inMatrix)->ToImage(true).save("ScriptFilterInput.png");
	QProcess::execute(m_sFilterType+" "+m_sParameters);	
	QImage result=QImage("ScriptFilterOutput.png");
	*outMatrix = new SymMatrix2DReal(result);
}

void SymFilterScript::Process(SymMatrix * inOutMatrix)
{
	assert(inOutMatrix->GetNumDims()==2 && inOutMatrix->GetEntryType()==SYM_ENTRY_REAL);
//	std::cout<<"min(mg)="<<inOutMatrix->GetMin(false)<<" max(mg)="<<inOutMatrix->GetMax(false)<<std::endl;
	((SymMatrix2DReal*)inOutMatrix)->ToImage(true).save("ScriptFilterInput.png");
	QProcess::execute(m_sFilterType+" "+m_sParameters);	
	QImage result=QImage("ScriptFilterOutput.png");
	assert(result.width()==((SymMatrix2DReal*)inOutMatrix)->m && result.height()==((SymMatrix2DReal*)inOutMatrix)->n);
	((SymMatrix2DReal*)inOutMatrix)->FromImage(result);
}

SymFilterGaussian::SymFilterGaussian(double stdDev)
{
	m_fStdDev = stdDev;
}

void SymFilterGaussian::Process(SymMatrix * inOutMatrix)
{
	assert(inOutMatrix->GetEntryType()==SYM_ENTRY_REAL && inOutMatrix->GetNumDims()==2);
	Process2DReal((SymMatrix2DReal*)inOutMatrix, m_fStdDev);
}

void SymFilterGaussian::Process2DReal(SymMatrix2DReal * inMatrix, double std_dev)
{
	inMatrix->GaussBlur(std_dev);	
}

SymFilterPntAbsPow::SymFilterPntAbsPow(double p, double maxVal, double cutRad)
{
	m_fPow = p;
	m_fMaxVal = maxVal;
	m_fCutRadius = cutRad;
}

void SymFilterPntAbsPow::Process(SymMatrix * inOutMatrix)
{
	assert(inOutMatrix->GetEntryType()==SYM_ENTRY_REAL && inOutMatrix->GetNumDims()==2);
	Process2DReal((SymMatrix2DReal*)inOutMatrix, m_fPow, m_fMaxVal, m_fCutRadius);	
}

void SymFilterPntAbsPow::Process2DReal(SymMatrix2DReal * inMatrix, double p, 
									   double maxVal, double maxRad)
{
	double cx = (double)inMatrix->m/2.;
	double cy = (double)inMatrix->n/2.;
	if (maxRad>0)
		maxRad*=maxRad;
	for (int i=0; i<inMatrix->m; i++)
	{
		for (int j=0; j<inMatrix->n; j++)
		{
			(*inMatrix)(i,j) = pow(qAbs((*inMatrix)(i,j)), p);
			if ((*inMatrix)(i,j)>maxVal && maxVal>0)
				(*inMatrix)(i,j) = maxVal;
	
			if (maxRad>0)
			{
				double dx = cx - i - 0.5;
				double dy = cy - j - 0.5;
				if (dx*dx+dy*dy < maxRad)
					(*inMatrix)(i,j) = 0;
			}
		}
	}
}

SymFilterGradient::SymFilterGradient(double std_dev, int window)
{
	m_fBlurStdDev = std_dev;
	m_iWindow = window;
}

void SymFilterGradient::Process(SymMatrix * inOutMatrix)
{
	assert(inOutMatrix->GetEntryType()==SYM_ENTRY_REAL && inOutMatrix->GetNumDims()==2);
	Process2DReal((SymMatrix2DReal*)inOutMatrix, m_fBlurStdDev, m_iWindow);
}

void SymFilterGradient::Process2DReal(SymMatrix2DReal * inOutMatrix, double blurStdDev, int window)
{	
//	inOutMatrix->ToImage().save("SobelInput.png");	
	std::cout<<"window="<<window<<std::endl;
	if (window <= 0 || window>31)	// GAPS
	{
		inOutMatrix->FillGrid();
		inOutMatrix->m_pTempFilterGrid->DetectEdges();
//		inOutMatrix->m_pTempFilterGrid->GradientMagnitude();
//		std::cout<<"blur="<<blurStdDev<<std::endl;
		if (blurStdDev>0)
			inOutMatrix->m_pTempFilterGrid->Blur(blurStdDev);
		inOutMatrix->CopyFromGrid();
	}
	else	// OpenCV
	{
		inOutMatrix->SobelFilterOpenCV(window);
		if (blurStdDev>0)
			inOutMatrix->BlurFilterOpenCV(blurStdDev);	
	}
//	inOutMatrix->ToImage().save("SobelOutput.png");	
}

SymFilterCanny::SymFilterCanny(double afterBlurStdDev, double threshold1, double threshold2, int window )
{
	m_fThrehsold1 = threshold1;
	m_fThrehsold2 = threshold2;
	m_fAfterBlur = afterBlurStdDev;
	m_iWindow = window;
	
}

void SymFilterCanny::Process(SymMatrix * inOutMatrix)
{
	assert(inOutMatrix->GetEntryType()==SYM_ENTRY_REAL && inOutMatrix->GetNumDims()==2);
	Process2DReal((SymMatrix2DReal *)inOutMatrix, m_fAfterBlur, m_fThrehsold1*m_iWindow, 
				  m_fThrehsold2*m_iWindow, m_iWindow);
}

void SymFilterCanny::Process2DReal(SymMatrix2DReal * inMatrix, double blurStdDev, 
								   double threshold1, double threshold2, int window)
{
	inMatrix->CannyFilterOpenCV(threshold1, threshold2, window);
	if (blurStdDev>0)
		inMatrix->BlurFilterOpenCV(blurStdDev);		
}


SymFilterSequence::SymFilterSequence()
{
}

SymFilterSequence::~SymFilterSequence()
{
}

int SymFilterSequence::GetNumFilters()
{
	return m_FilterSequence.size();
}

SymFilter * SymFilterSequence::GetFilter(int fltID)
{
	assert(fltID < (int)m_FilterSequence.size());
	if (fltID<0)
		return m_FilterSequence[(int)m_FilterSequence.size()-1];
	else
		return m_FilterSequence[fltID];
}

void SymFilterSequence::AddFilter(SymFilter * filter, int id)
{
	assert(id<=(int)m_FilterSequence.size());
	if (id==-1)
		m_FilterSequence.push_back(filter);
	else
		m_FilterSequence[id] = filter;
}

void SymFilterSequence::Process(SymMatrix * inOutMatrix)
{
	for (int i=0; i<(int)m_FilterSequence.size(); i++)
		m_FilterSequence[i]->Process(inOutMatrix);
}





