#include "SymPipeline.h"
#include <Qt/qimage.h>
#include "DeformImageWarping.h"
#include <Qt/qprocess.h>
#include <Qt/qfile.h>
#include <Qt/qtextstream.h>
#include "TimeProfiler.h"


SymPipeline::SymPipeline(SymPipelineParams * params)
{
	m_pOptFnParams = params;
}

SymPipeline::~SymPipeline()
{
}

void SymPipeline::FindOptimalSolution()
{	
	SymOptimizer * optimizer = m_pOptFnParams->m_pOptimizer;
	optimizer->Iterate();
	double penalty;	
	optimizer->m_pDeformationModel->DeformAtCurrentHierarchyLevel(*(optimizer->GetBestSolution()), &penalty);
}

void SymPipeline::OutputResultsImg(QString outDir, QImage * f, SymMatrix2DReal * Sf, QImage * g, 
								   SymMatrix2DReal * Sg, QString fileExt, bool printHistoryGifs)
{
	SymOptimizer * optimizer = m_pOptFnParams->m_pOptimizer;
//#ifdef SYM_MACOS
	QProcess::execute("rm -R "+outDir);
	QProcess::execute("mkdir "+outDir);
	if (printHistoryGifs)
	{
		for (int i=0; i<(int)optimizer->m_HistoryLandmark.size(); i++)
		{
			QProcess::execute("convert "+QString("progress_f_")+AddTrailingZeros(i, 6)+".png " + QString("progress_f_")+AddTrailingZeros(i, 6)+".gif");
			QProcess::execute("convert "+QString("progress_SfVis_")+AddTrailingZeros(i, 6)+".png " + QString("progress_Sf_")+AddTrailingZeros(i, 6)+".gif");			
		}
		QProcess::execute("cp "+QString("targetSymmetryVis.png ")+outDir+"/SgVis.png");
		
		//QProcess::execute("gifsicle -d 0 -l ./progress_f*.gif > ./"+outDir+"/anim_f.gif");
		//QProcess::execute("gifsicle -d 0 -l ./progress_Sf*.gif > ./"+outDir+"/anim_Sf.gif");
		//QProcess::execute("rm progress*");
	}
	f->save(outDir + "/f." + fileExt);
	SymSpace * S = m_pOptFnParams->m_pEnergyFn->m_pS;
	double penalty;
	SymVectorReal * bestSol = optimizer->GetBestSolution();
	SymMatrix & sol = optimizer->m_pDeformationModel->DeformAtCurrentHierarchyLevel(*bestSol, &penalty);
	if (optimizer->m_pDeformationModel->HasOutput(DeformationModel::DEFORM_OUT_IMG2D))
	{
		((SymMatrix2DReal&)(S->CalculateSymmetryAndProcess(sol))).ToImage().save(outDir + "/Sfp." + fileExt);		
		optimizer->m_pDeformationModel->GetDeformedImage(DeformationModel::DEFORMATION_FINAL_RESULT).save(outDir + "/fp." + fileExt);
		for (int i=0; i<optimizer->m_pDeformationModel->GetNumOutputImageTypes(); i++)
			optimizer->m_pDeformationModel->GetDeformedImage(i).save(outDir + "/fp"+QString::number(i)+"." + fileExt);
	}
	
	if (Sf!=NULL)
		Sf->ToImage().save(outDir + "/Sf." + fileExt);
	else 
	{
		SymMatrix2DReal fMtrx(*f);
		if (((SymSpace2D*)S)->image_m==fMtrx.m && ((SymSpace2D*)S)->image_n==fMtrx.n)
		{
			((SymMatrix2DReal&)(S->CalculateSymmetryAndProcess(fMtrx))).ToImage().save(outDir+"/Sf."+fileExt);
		}
	}

	if (g!=NULL)
		g->save(outDir + "/g." + fileExt);

	if (Sg!=NULL)
	{	
		Sg->ToImage().save(outDir + "/Sg." + fileExt);
	}
	else if (g!=NULL)
	{
		SymMatrix2DReal gMtrx(*g);		
		((SymMatrix2DReal&)(S->CalculateSymmetryAndProcess(gMtrx))).ToImage().save(outDir+"/Sg."+fileExt);
	}

	QFile histFile(outDir+"/history.txt");
	
	if (histFile.open(QIODevice::WriteOnly))
	{
		QTextStream outFileStream(&histFile);
		if ((int)optimizer->m_EnergyHistory.size()>0)
			outFileStream<<"Experiment output: E_final="<<optimizer->m_EnergyHistory[(int)optimizer->m_EnergyHistory.size()-1]<<"\n";
		for (int i=0; i<(int)optimizer->m_HistoryLandmark.size(); i++)	//print landmarks
		{
			int L = optimizer->m_HistoryLandmark[i];
			outFileStream<<"Iteration "<<L<<": E="<<optimizer->m_EnergyHistory[L]<<" dx="<<optimizer->m_DXStepHistory[L]<<" FnEvals="<<optimizer->m_iFnEvalsHistory[L]<<"\n";
		}
		outFileStream.flush();
	}
	else
	{
		std::cout<<"SymPipeline::OutputResultsImg Could not write "<<(outDir+"/history.txt").toStdString().c_str()<<std::endl;
	}
	
	histFile.close();
	QFile eHistFile(outDir+"/energyMatlab.txt");
	
	if (eHistFile.open(QIODevice::WriteOnly))
	{
		QTextStream outFileStream(&eHistFile);
		outFileStream<<"EnergyHistory = [ ";
		for (int i=0; i<(int)optimizer->m_HistoryLandmark.size(); i++)	//print landmarks only
			outFileStream<<optimizer->m_EnergyHistory[optimizer->m_HistoryLandmark[i]]<<" ";
		outFileStream<<"];\n";
		
		if (optimizer->m_pAnswer!=NULL)	//answer is known
		{
			outFileStream<<"ImageErrors = [ ";		
			for (int i=0; i<(int)optimizer->m_ImageErrors.size(); i++)	//print landmarks only
				outFileStream<<optimizer->m_ImageErrors[i]<<" ";
			outFileStream<<"];\n";
			
			outFileStream<<"ParamErrors = [ ";		
			for (int i=0; i<(int)optimizer->m_ParamErrors.size(); i++)	//print landmarks only
				outFileStream<<optimizer->m_ParamErrors[i]<<" ";
			outFileStream<<"];\n";
			
			outFileStream<<"PerfectSymEnergy="<<optimizer->m_pAnswer->m_fEnergyPerfect<<"\n";
			outFileStream<<"PerfectDeformSymEnergy="<<optimizer->m_pAnswer->m_fEnergyDeform<<"\n";
			
			optimizer->m_pAnswer->m_ImagePerfectOutput.save(outDir + "/Perfect." + fileExt);
			optimizer->m_pAnswer->m_ImagePerfectDeformed.save(outDir + "/PerfectDeformed." + fileExt);			
		}
		
		outFileStream.flush();
	}
	else
	{
		std::cout<<"SymPipeline::OutputResultsImg Could not write "<<(outDir+"/energyMatlab.txt").toStdString().c_str()<<std::endl;
	}	
	eHistFile.close();
}

QString SymPipeline::AddTrailingZeros(int val, int numChars)
{
	QString newStr = QString::number(val);
	while(newStr.count()<numChars)
		newStr = "0"+newStr;
	return newStr;
}





