
#include "Numerical.h"
#include "SymFilters.h"
#include "TimeProfiler.h"

#ifndef __SYM_SPACE_H
#define __SYM_SPACE_H

/**
 * General Symmetry Space
 */
class SymSpace
{
	public:
		enum NormalizationType
		{
			NORM_NONE,
			NORM_SYMMETRY_SPACE,
			NORM_IMAGE_SPACE,
			NORM_GAUSS_MEAN0_STD1,
			NORM_SYMMETRY_SPACE_MAX_TO_1
		};
	
//		struct FilterParams
//		{
//			//FilterParams(char ** argv, int argc, int & argID);
//			void InitGaussBlur(double sigma);
//			void InitExpSharpen(double p);
//			QString m_sName;
//			double m_fPow;
//			double m_fBlurStdDev;
//		};
	
	
		SymSpace(double p, NormalizationType norm, double constNormalizationScalar, 
				 int iterativeApplication = 1, bool cropToDisk = false);
		virtual ~SymSpace();
		virtual SymMatrix & CalculateSymmetryAndProcess(SymMatrix & f, bool normalize=true, double * L2f = NULL);
	
		virtual void Normalize(SymMatrix & Sf, double L2f);
	
		//NOTE: if f has to be pre-filtered (if needed), this function only fitlers Sf
		virtual SymMatrix * GetFilteredSymSpace(SymMatrix * f, SymFilter * symSpaceFilter);
		
		virtual SymMatrix * RescaleAndNormalize(SymMatrix * Sf);
		virtual void Renormalize(SymMatrix * Sf);
	
		SymMatrix * m_pSymmetryValues;	
		NormalizationType m_Norm;
	
		static FFTManager m_FFTManager;		
		double p;	
		double m_fConstNormalizationScalar;
		int m_iIterativeSymmetry;

	
		virtual void SetCenterOfMass();
		virtual void CropToDisk(SymMatrix2DReal & f);
		bool m_bCropToDisk;
		R2Point m_Center;
	
		virtual void SetVisualizationParams(std::vector<int> &colors, std::vector<double> & crop, double gamma);	
	
	protected:
		int m_iVisR, m_iVisG, m_iVisB;
		double m_fVisCropMin, m_fVisCropMax;
		double m_fVisAttenuateGamma;
	
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f) = 0;
		virtual void CalculateSymmetryMultiIter(SymMatrix & f);
};



/**
 * 2D Symmetry space
 */
class SymSpace2D : public SymSpace
{
	public:	
		SymSpace2D(int im_m, int im_n, int s_m, int s_n, double p, NormalizationType norm, 
				   double constNormalizationScalar, int iterativeApplication = 1, bool cropToDisk = false);
		virtual QImage GetVisualizationOfSymSpace(SymMatrix2DReal & Sf, QImage * src = NULL);	
		virtual ~SymSpace2D();
		
		void CropCenter(SymMatrix2DReal & source);
		void CropCorners(SymMatrix2DReal & source);
		

		int image_m, image_n;
		int symm_m, symm_n;
};

/**
 * Identity - no change
 */
class SymSpaceIdentity : public SymSpace2D
{
	public:
		SymSpaceIdentity(int dimW, int dimH, NormalizationType normType);
		virtual ~SymSpaceIdentity(){}
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f);
};		

/**
 * 2D translational symmetry (for toroidal functions)
 */
class SymSpaceTransToroidal2D : public SymSpace2D
{
	public:
		SymSpaceTransToroidal2D(int im_m, int im_n, int S_m=-1, int S_n=-1, 
								NormalizationType norm = SymSpace::NORM_IMAGE_SPACE, double p = 1, 
								double constNormalizationScalar = 1, int iterativeApplication = 1);
		virtual SymMatrix * RescaleAndNormalize(SymMatrix * Sf);
	
		virtual ~SymSpaceTransToroidal2D();
	
		static void TestSymmetryTransform();
	
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f);	
		virtual void CalculateSymmetryMultiIter(SymMatrix & f);	
		SymMatrix2DComplex * m_pTmpFFTConj;
		SymMatrix2DComplex * m_pTmpFFT;
		SymMatrix2DComplex * m_pTmpConvFFT;
		SymMatrix2DReal	* m_pTmpFullAnswer;
};

/**
 * 2D translational symmetry
 */
class SymSpaceTrans2D : public SymSpace2D
{
	public:	
		SymSpaceTrans2D(int im_m, int im_n, int S_m=-1, int S_n=-1, NormalizationType norm = SymSpace::NORM_IMAGE_SPACE, double p = 1, double constNormalizationScalar = 1);
		virtual ~SymSpaceTrans2D();
	
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f);	
		SymMatrix2DComplex * m_pTmpFFTConj;
		SymMatrix2DComplex * m_pTmpFFT;
		SymMatrix2DComplex * m_pTmpConvFFT;
		SymMatrix2DReal	* m_pTmpFullAnswer;
		SymMatrix2DReal * m_pPaddedF1;
		SymMatrix2DReal * m_pPaddedF2;	
	
};

//class SymSpaceTransCrossCorr2D : public SymSpace2D
//{
//	public:
//		SymSpaceTransCrossCorr2D(int im_m, int im_n, int S_m=-1, int S_n=-1, 
//								 NormalizationType norm = SymSpace::NORM_IMAGE_SPACE);
//	protected:
//		virtual void CalculateSymmetry(SymMatrix & f);
//};
//
//class SymSpaceCrossCorrCollection : public SymSpace
//{
//	public:
//		SymSpaceCrossCorrCollection(std::vector<SymSpace *> & collectionOfSymSpaces);
//};

#endif



