
#include "Numerical.h"
#include "SymFilters.h"
#include "TimeProfiler.h"
#include "SymSpace.h"
#include "SymSpaceReflective.h"

#ifndef __SYM_SPACE_ADDITIONAL_H
#define __SYM_SPACE_ADDITIONAL_H

/**
 * Pointwise Symmetry Space: Toroidal
 */
class SymSpacePointwiseToroidal : public SymSpace2D
{
	public:
		SymSpacePointwiseToroidal(int im_m, int im_n, int S_m=-1, int S_n=-1, 
						  NormalizationType norm = SymSpace::NORM_IMAGE_SPACE);
		virtual ~SymSpacePointwiseToroidal(){}
		
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double *L2f);
//		SymMatrix2DComplex * m_pTmpFFTConj;
		SymMatrix2DComplex * m_pTmpFFT;
		SymMatrix2DComplex * m_pTmpConvFFT;
		SymMatrix2DReal	* m_pTmpFullAnswer;
	
};

/**
 * Pointwise Symmetry Space: Padded
 */
class SymSpacePointwisePadded : public SymSpace2D
{
	public:
		SymSpacePointwisePadded(int im_m, int im_n, int S_m=-1, int S_n=-1, 
								NormalizationType norm = SymSpace::NORM_IMAGE_SPACE);
		virtual ~SymSpacePointwisePadded(){}
	
		static void TestPointwise();	
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f);
		SymMatrix2DReal * m_pPaddedF;
		SymMatrix2DComplex * m_pTmpFFT;
		SymMatrix2DComplex * m_pTmpConvFFT;
		SymMatrix2DReal	* m_pTmpFullAnswer;	
};

/**
 * Rotational Symmetry Space
 */
class SymSpaceRotationalSinglePnt2D : public SymSpaceSinglePointReflection2D
{
	public:
		SymSpaceRotationalSinglePnt2D(int im_m, int im_n, int numAngles=-1, int numRadii=-1, 
						   DeformationModel * deformationModel = NULL, 
						   NormalizationType norm = SymSpace::NORM_IMAGE_SPACE,
						   bool cropToDisk=true, int centerOfMass = SymSpaceSinglePointReflection2D::COM_AREA_MASS,
						   double ignoreMinAngle=0, double ignoreMaxAngle=0);
		virtual ~SymSpaceRotationalSinglePnt2D(){}
		static void FillColSymmetryPerAngleAutocorr(SymMatrix2DReal & fillMe, int col, 
													SymMatrix2DReal & f_polar, 
													SymMatrix2DComplex * tmpFFT, 
													SymMatrix2DComplex * tmpConvFFT, 
													SymMatrix2DComplex * tmpConjFFT);
		
	protected:
		virtual void CalculateSymmetry(SymMatrix & f, double * L2f);
		SymMatrix2DComplex * m_pTmpConjFFT;
		int m_iIgnoreMin;
		int m_iIgnoreMax;
		double m_fSetToValue;
};

#endif



