/*

File: example.cpp

An example of the use of the Quasisampler class.
Reminder: This is a toy implementation, created
to aid in understanding how the system works.

This example generates a set of points for a 
constant-value function.

Usage: example [magnitude_factor=1000]

This is a toy (non-optimized) implementation of the importance sampling
technique proposed in the paper:
"Fast Hierarchical Importance Sampling with Blue Noise Properties",
by Victor Ostromoukhov, Charles Donohue and Pierre-Marc Jodoin,
to be presented at SIGGRAPH 2004.


Implementation by Charles Donohue,
Based on Mathematica code by Victor Ostromoukhov.
Universite de Montreal
05.08.04

*/

#include <iostream>
#include "quasisampler_prototype.h"


class MyQuasisampler : public Quasisampler
{
  unsigned const_val;

public:
  MyQuasisampler(int numPnts, unsigned screenSize) : 
	Quasisampler(screenSize, screenSize), 
	const_val(numPnts*10) {}

  unsigned getImportanceAt( Point2D pt ) 
  { 
    return const_val; // This function returns a constant value.
  }
};


typedef std::vector<Point2D> PointList;

int main(int argc, char* argv[])
{
	assert(argc==4);
	// initialize sampler
	//screenSize, numPnts
	MyQuasisampler test(atoi(argv[1]), atoi(argv[2]));
	// generate points
	PointList points = test.getSamplingPoints();

	int numPnts = 0;
	for ( PointList::iterator it=points.begin(); it!=points.end(); it++ )
		numPnts++;
	
	FILE * outfile = fopen(argv[3], "w");
	std::cout<<"OutFile = "<<argv[3]<<std::endl;
	assert(outfile!=NULL);
	
	fprintf(outfile, "%%!PS-Adobe EPSF-3.0\n%%%%HiResBoundingBox: 0 0 %d %d\n", atoi(argv[2]), atoi(argv[2]));
	fprintf(outfile, "%%%%BoundingBox: 0 0 %d %d\n%%%% Sites: %d\n/radius { 2 } def\n", atoi(argv[2]), atoi(argv[2]), numPnts);
	fprintf(outfile, "/p { radius 0 360 arc closepath fill } def\n0 0 0 setrgbcolor\n\n");
	
	// print points
	for ( PointList::iterator it=points.begin(); it!=points.end(); it++ )
	{
		fprintf(outfile, "%f %f p\n", it->x, it->y);
		//std::cout << it->x << "," << it->y << std::endl;
	}
	
	fprintf(outfile, "showpage");
	
	std::cout<<"numPnts="<<numPnts<<std::endl;	
	
	fclose(outfile);
	
	return 0;
}

